116 lines
3.5 KiB
C++
116 lines
3.5 KiB
C++
// Copyright (c) 2010-2020, Lawrence Livermore National Security, LLC. Produced
|
|
// at the Lawrence Livermore National Laboratory. All Rights reserved. See files
|
|
// LICENSE and NOTICE for details. LLNL-CODE-806117.
|
|
//
|
|
// This file is part of the MFEM library. For more information and source code
|
|
// availability visit https://mfem.org.
|
|
//
|
|
// MFEM is free software; you can redistribute it and/or modify it under the
|
|
// terms of the BSD-3 license. We welcome feedback and contributions, see file
|
|
// CONTRIBUTING.md for details.
|
|
|
|
#ifndef MFEM_CUDA_HPP
|
|
#define MFEM_CUDA_HPP
|
|
|
|
#include "../config/config.hpp"
|
|
#include "error.hpp"
|
|
|
|
#ifdef MFEM_USE_CUDA
|
|
#include <cuda_runtime.h>
|
|
#include <cuda.h>
|
|
#endif
|
|
|
|
// CUDA block size used by MFEM.
|
|
#define MFEM_CUDA_BLOCKS 256
|
|
|
|
#ifdef MFEM_USE_CUDA
|
|
#define MFEM_DEVICE __device__
|
|
#define MFEM_HOST_DEVICE __host__ __device__
|
|
// Define a CUDA error check macro, MFEM_GPU_CHECK(x), where x returns/is of
|
|
// type 'cudaError_t'. This macro evaluates 'x' and raises an error if the
|
|
// result is not cudaSuccess.
|
|
#define MFEM_GPU_CHECK(x) \
|
|
do \
|
|
{ \
|
|
cudaError_t err = (x); \
|
|
if (err != cudaSuccess) \
|
|
{ \
|
|
mfem_cuda_error(err, #x, _MFEM_FUNC_NAME, __FILE__, __LINE__); \
|
|
} \
|
|
} \
|
|
while (0)
|
|
#define MFEM_DEVICE_SYNC MFEM_GPU_CHECK(cudaDeviceSynchronize())
|
|
#define MFEM_STREAM_SYNC MFEM_GPU_CHECK(cudaStreamSynchronize(0))
|
|
#endif // MFEM_USE_CUDA
|
|
|
|
// Define the MFEM inner threading macros
|
|
#if defined(MFEM_USE_CUDA) && defined(__CUDA_ARCH__)
|
|
#define MFEM_SHARED __shared__
|
|
#define MFEM_SYNC_THREAD __syncthreads()
|
|
#define MFEM_THREAD_ID(k) threadIdx.k
|
|
#define MFEM_THREAD_SIZE(k) blockDim.k
|
|
#define MFEM_FOREACH_THREAD(i,k,N) for(int i=threadIdx.k; i<N; i+=blockDim.k)
|
|
#endif
|
|
|
|
#if !(defined(MFEM_USE_CUDA) || defined(MFEM_USE_HIP))
|
|
#define MFEM_DEVICE
|
|
#define MFEM_HOST_DEVICE
|
|
#define MFEM_DEVICE_SYNC
|
|
#define MFEM_STREAM_SYNC
|
|
#endif
|
|
|
|
#if !((defined(MFEM_USE_CUDA) && defined(__CUDA_ARCH__)) || \
|
|
(defined(MFEM_USE_HIP) && defined(__ROCM_ARCH__)))
|
|
#define MFEM_SHARED
|
|
#define MFEM_SYNC_THREAD
|
|
#define MFEM_THREAD_ID(k) 0
|
|
#define MFEM_THREAD_SIZE(k) 1
|
|
#define MFEM_FOREACH_THREAD(i,k,N) for(int i=0; i<N; i++)
|
|
#endif
|
|
|
|
namespace mfem
|
|
{
|
|
|
|
#ifdef MFEM_USE_CUDA
|
|
// Function used by the macro MFEM_GPU_CHECK.
|
|
void mfem_cuda_error(cudaError_t err, const char *expr, const char *func,
|
|
const char *file, int line);
|
|
#endif
|
|
|
|
/// Allocates device memory and returns destination ptr.
|
|
void* CuMemAlloc(void **d_ptr, size_t bytes);
|
|
|
|
/// Allocates managed device memory
|
|
void* CuMallocManaged(void **d_ptr, size_t bytes);
|
|
|
|
/// Frees device memory and returns destination ptr.
|
|
void* CuMemFree(void *d_ptr);
|
|
|
|
/// Copies memory from Host to Device and returns destination ptr.
|
|
void* CuMemcpyHtoD(void *d_dst, const void *h_src, size_t bytes);
|
|
|
|
/// Copies memory from Host to Device and returns destination ptr.
|
|
void* CuMemcpyHtoDAsync(void *d_dst, const void *h_src, size_t bytes);
|
|
|
|
/// Copies memory from Device to Device
|
|
void* CuMemcpyDtoD(void *d_dst, const void *d_src, size_t bytes);
|
|
|
|
/// Copies memory from Device to Device
|
|
void* CuMemcpyDtoDAsync(void *d_dst, const void *d_src, size_t bytes);
|
|
|
|
/// Copies memory from Device to Host
|
|
void* CuMemcpyDtoH(void *h_dst, const void *d_src, size_t bytes);
|
|
|
|
/// Copies memory from Device to Host
|
|
void* CuMemcpyDtoHAsync(void *h_dst, const void *d_src, size_t bytes);
|
|
|
|
/// Check the error code returned by cudaGetLastError(), aborting on error.
|
|
void CuCheckLastError();
|
|
|
|
/// Get the number of CUDA devices
|
|
int CuGetDeviceCount();
|
|
|
|
} // namespace mfem
|
|
|
|
#endif // MFEM_CUDA_HPP
|